/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.filechooser.FileFilter;

import ca.uvic.csr.shrimp.AttrToVisVarBean.LabelStyleVisualVariable;
import ca.uvic.csr.shrimp.AttrToVisVarBean.NodeShapeVisualVariable;
import ca.uvic.csr.shrimp.DisplayBean.NodeShape.NodeImage;
import ca.uvic.csr.shrimp.DisplayBean.NodeShape.NodeShape;
import ca.uvic.csr.shrimp.gui.ActionManager.MenuAction;
import ca.uvic.csr.shrimp.gui.ActionManager.ShrimpAction;
import ca.uvic.csr.shrimp.resource.ResourceHandler;

/**
 * Creates a dialog for choosing the color and style of an arc.
 * 
 * @author Chris Callendar
 */
public class NodePresentationDialog extends StylePresentationDialog {
	
	private static final NodeShape DEFAULT_SHAPE = NodeShapeVisualVariable.DEFAULT_NODE_SHAPE;
	private static final String DEFAULT_STYLE = LabelStyleVisualVariable.DEFAULT_LABEL_STYLE;
	
	private NodeShape nodeShape;
	private Vector nodeShapes;
	private String labelStyle;
	private Vector labelStyles;
	private Vector imageModes;
	private NodeImage nodeImage;
	private final Color defaultColor;
	
	private ShrimpAction loadAction;
	private ShrimpAction deleteAction;

	private JFileChooser chooser;
	private JComboBox cmbShapes;
	private JLabel lblImageName;
	private JCheckBox chkFillBackground;
	private JCheckBox chkDrawBorder;
	private JComboBox cmbImageSizing;
	private JComboBox cmbLabelStyle;
	
	/**
	 * Creates a NodePresentationDialog.
	 * @param owner The owner of this dialog.
	 * @param nodeShapes The available {@link NodeShape}s to choose from.
	 * @param nodeShape The current node shape.
	 * @param color The current node color.
	 * @param defaultColor the default node color
	 * @param nodeImage the image and settings
	 * @param relativeTo The component to position this dialog relative to.
	 */
	public NodePresentationDialog(Frame owner, Vector nodeShapes, NodeShape nodeShape, Vector labelStyles,
			String labelStyle, Color color, Color defaultColor, NodeImage nodeImage, Component relativeTo) {
		super(owner, "Node Style", color);
		
		this.nodeShapes = filterShapes(nodeShapes);
		this.nodeShape = nodeShape;
		this.labelStyles = labelStyles;
		this.labelStyle = labelStyle;
		this.defaultColor = defaultColor;
		this.nodeImage = nodeImage;
		
		imageModes = new Vector(4);
		imageModes.add(NodeImage.NO_SCALING);
		imageModes.add(NodeImage.STRETCHED);
		imageModes.add(NodeImage.CENTERED);
		imageModes.add(NodeImage.TILED);	
		
		createGUI(relativeTo);	// blocks
	}
	
	/**
	 * Removes any node shapes that aren't user selectable.
	 * @return a new Vector containing all the user selectable shapes.
	 */
	protected static Vector filterShapes(Vector nodeShapes) {
		Vector shapes = new Vector(nodeShapes.size());
		for (Iterator iter = nodeShapes.iterator(); iter.hasNext(); ) {
			NodeShape shape = (NodeShape) iter.next();
			if (shape.isUserSelectable()) {
				shapes.add(shape);
			}
		}
		return shapes;
	}
	
	private JFileChooser getFileChooser() {
		if (chooser == null) {
			chooser = new JFileChooser();
			chooser.setDialogTitle("Choose an image");
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileFilter(new FileFilter() {
				public boolean accept(File f) {
					String name = f.getName().toLowerCase();
					boolean isImg = name.endsWith(".jpg") || name.endsWith(".jpeg") || 
						name.endsWith(".bmp") || name.endsWith(".gif") || name.endsWith(".png");
					return isImg || f.isDirectory();
				}
				public String getDescription() {
					return "Image Files (*.jpg, *.gif, *.png, *.bmp)";
				}
			});
			if (nodeImage.getImagePath().length() > 0) {
				chooser.setSelectedFile(new File(nodeImage.getImagePath()));
			}
		}
		return chooser;
	}
	
	protected void addRows() {
		createNodeShapePanel();
		createLabelStylesPanel();
		createImagePanel();
		createCheckBoxRows();
		createPreviewPanel();
		updateImageProperties();
		updateThumbnail();
	}

	/**
	 * Creates the components for choosing a node shape.
	 */
	private void createNodeShapePanel() {
		cmbShapes = new JComboBox(nodeShapes);
		cmbShapes.setSelectedItem(nodeShape);
		cmbShapes.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					nodeShape = (NodeShape) e.getItem();
					updateThumbnail();
				}
			}
		});
		addRow("Node Shape", cmbShapes);
	}
	
	/**
	 * Creates the components for choosing the node label style.
	 */
	private void createLabelStylesPanel() {
		cmbLabelStyle = new JComboBox(labelStyles);
		cmbLabelStyle.setSelectedItem(labelStyle);
		cmbLabelStyle.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					labelStyle = (String) e.getItem();
				}
			}
		});
		addRow("Label Style", cmbLabelStyle);
	}

	/**
	 * Creates the panel with the image settings.
	 */
	private void createImagePanel() {
		String imagePath = nodeImage.getImagePath();
		File imageFile = new File(imagePath);
		
		final JPanel panel = new JPanel(new BorderLayout());
		panel.setOpaque(false);
		JToolBar toolbar = createImageToolBar(imagePath, imageFile);
		panel.add(toolbar, BorderLayout.CENTER);
		addRow("Image", panel);
		
		cmbImageSizing = new JComboBox(imageModes);
		cmbImageSizing.setSelectedItem(nodeImage.getImageSizing());
		cmbImageSizing.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					nodeImage.setImageSizing((String) e.getItem());
				}
			}
		});
		addRow("Sizing", cmbImageSizing);
		
		loadAction = new MenuAction(ResourceHandler.getIcon("icon_open.gif")) {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = getFileChooser();
				if (fileChooser.showOpenDialog(panel) == JFileChooser.APPROVE_OPTION) {
					File file = chooser.getSelectedFile();
					String imagePath = file.getAbsolutePath();
					lblImageName.setText(file.getName());
					lblImageName.setToolTipText(imagePath);
					nodeImage.setImagePath(imagePath);
					updateImageProperties();
					updateThumbnail();
					
					// this is done to force the image to show up
					// it seems to take a few repaints before it appears
					new Thread(new Runnable() {
						public void run() {
							// this is done twice to ensure that the image does show up
							for (int i = 0; i < 2; i++) {
								try {
									Thread.sleep(500);
								} catch (InterruptedException e) {}
								pnlPreview.repaint();
							}
						}
					}).start();
				}
			}
		};
		loadAction.setToolTip("Load an image for this node type");
		toolbar.add(loadAction).setOpaque(false);
		
		deleteAction = new MenuAction(ResourceHandler.getIcon("icon_delete.gif")) {
			public void actionPerformed(ActionEvent e) {
				nodeImage.setImagePath("");
				lblImageName.setText("");
				lblImageName.setToolTipText("");
				updateImageProperties();
				updateThumbnail();
			}
		};
		deleteAction.setToolTip("Remove the image from this node type");
		toolbar.add(deleteAction).setOpaque(false);
	}

	/**
	 * @param imagePath
	 * @param imageFile
	 */
	private JToolBar createImageToolBar(String imagePath, File imageFile) {
		JToolBar toolbar = new JToolBar();
		toolbar.setOpaque(false);
		toolbar.setFloatable(false);
		lblImageName = new JLabel(imageFile.exists() ? imageFile.getName() : " ");
		lblImageName.setToolTipText(imagePath);
		lblImageName.setOpaque(true);
		lblImageName.setBackground(Color.white);
		lblImageName.setPreferredSize(new Dimension(100, 16));
		lblImageName.setMinimumSize(new Dimension(100, 16));
		lblImageName.setMaximumSize(new Dimension(160, 24));
		lblImageName.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLoweredBevelBorder(), 
				BorderFactory.createEmptyBorder(0, 2, 0, 2)));
		toolbar.add(lblImageName);
		toolbar.add(new JLabel(" "));
		return toolbar;
	}
	
	private void createCheckBoxRows() {
		chkDrawBorder = new JCheckBox("", nodeImage.isDrawBorder());
		chkDrawBorder.setOpaque(false);
		addRow("Draw Border?", chkDrawBorder);

		JPanel pnl = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		pnl.setOpaque(false);
		chkFillBackground = new JCheckBox("", nodeImage.isFillBackground());
		chkFillBackground.setOpaque(false);
		pnl.add(chkFillBackground);
		pnl.add(Box.createHorizontalStrut(8));
		pnl.add(createColorChooserLabel());
		addRow("Fill Background?", pnl);
		
		ItemListener itemListener = new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				nodeImage.setFillBackground(chkFillBackground.isSelected());
				nodeImage.setDrawBorder(chkDrawBorder.isSelected());
				updateThumbnail();
			}
		};
		chkFillBackground.addItemListener(itemListener);
		chkDrawBorder.addItemListener(itemListener);
	}

	protected void updateImageProperties() {
		boolean canChange = !nodeShape.hasCustomRendering();
		cmbImageSizing.setEnabled(canChange && nodeImage.hasImage());
		loadAction.setEnabled(canChange);
		deleteAction.setEnabled(canChange && nodeImage.hasImage());
		chkDrawBorder.setEnabled(canChange);
		chkFillBackground.setEnabled(canChange);
	}
	
	protected void setDefaults() {
		cmbShapes.setSelectedItem(DEFAULT_SHAPE);
		cmbLabelStyle.setSelectedItem(DEFAULT_STYLE);
		setColor(defaultColor);
		
		nodeImage.reset();
		lblImageName.setText("");
		lblImageName.setToolTipText("");
		cmbImageSizing.setSelectedItem(nodeImage.getImageSizing());
		chkFillBackground.setSelected(true);
		chkDrawBorder.setSelected(true);
		
		updateImageProperties();
	}
		
	protected void updateThumbnail() {
		pnlPreview.removeAll();
		JComponent thumbnail = nodeShape.getThumbnail(0, 0, 21, 21, getColor(), nodeImage);
		thumbnail.setOpaque(false);
		pnlPreview.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		pnlPreview.setPreferredSize(new Dimension(33, 33));
		pnlPreview.add(thumbnail, BorderLayout.CENTER);
		pnlPreview.validate();
		pnlPreview.repaint();
	}
	
	/** 
	 * Returns the node shape selected.
	 * This is only used when node shapes are displayed. 
	 */
	public NodeShape getNodeShape() {
		return nodeShape;
	}
	
	/**
	 * Returns the selected node label style.
	 * @return String label style
	 */
	public String getLabelStyle() {
		return labelStyle;
	}
		
	public NodeImage getNodeImage() {
		return nodeImage;
	}
	
}
